@@ -33,7 +33,11 @@
 
 
 typedef struct MedusaHash MedusaHash;
-typedef unsigned int MedusaToken;
+
+#ifndef MEDUSA_TOKEN_DEFINED
+typedef guint32 MedusaToken;
+#define MEDUSA_TOKEN_DEFINED
+#endif
 
 struct MedusaHash {
 
