@@ -33,10 +33,10 @@
 typedef struct MedusaReadLock MedusaReadLock;
 typedef struct MedusaWriteLock MedusaWriteLock;
 
-MedusaReadLock  *medusa_read_lock_get      (const char      *file_name);
+MedusaReadLock  *medusa_read_lock_get      (const char      *index_name);
 void             medusa_read_lock_release  (MedusaReadLock  *lock);
 
-MedusaWriteLock *medusa_write_lock_get     (const char      *file_name);
+MedusaWriteLock *medusa_write_lock_get     (const char      *index_name);
 void             medusa_write_lock_release (MedusaWriteLock *lock);
 
 
