@@ -178,6 +178,12 @@ medusa_master_db_create_index (const char *root_uri,
 	g_return_if_fail (root_uri != NULL);
 	g_return_if_fail (index_name != NULL);
 
+	medusa_unsearched_locations_initialize ();
+
+	if (medusa_is_unsearched_location (root_uri)) {
+		return;
+	} 
+
 	master_db = g_new0 (MedusaMasterDB, 1);
 
 	root_directory = gnome_vfs_get_local_path_from_uri (root_uri);
@@ -201,8 +207,6 @@ medusa_master_db_create_index (const char *root_uri,
 	master_db->log_level = log_level;
 	master_db->ref_count = 1;
 
-	medusa_unsearched_locations_initialize ();
-
 	/* Make sure that the databases are empty before we start */
 	g_assert (medusa_uri_list_number_of_uris (master_db->uri_list) == 0);
 	g_assert (medusa_file_system_db_get_number_of_records (master_db->file_system_db) == 0);
@@ -368,8 +372,8 @@ index_directory (MedusaMasterDB *master_db,
 			continue;
 		}
 		
-		file_uri = medusa_full_uri_from_directory_and_file (directory_uri,
-								    directory_entry->d_name);
+		file_uri = medusa_full_uri_from_directory_uri_and_file_name (directory_uri,
+									     directory_entry->d_name);
 		if (medusa_is_unsearched_location (file_uri)) {
 			log_message = g_strdup_printf ("Skipping file %s because it is on the stop list", file_uri);
 			medusa_log_event (log_message, MEDUSA_DB_LOG_ABBREVIATED);
@@ -429,8 +433,8 @@ index_file (MedusaMasterDB *master_db,
 	medusa_file_system_db_index_file (master_db->file_system_db, 
 					  uri_number,
 					  file_info);
-	uri = medusa_full_uri_from_directory_and_file (directory_uri,
-						       file_name);
+	uri = medusa_full_uri_from_directory_uri_and_file_name (directory_uri,
+								file_name);
 #ifdef TEXT_INDEX_ON
 	medusa_text_index_read_file (master_db->text_index,
 				     uri,
@@ -714,7 +718,7 @@ add_public_keywords_to_index (const char *file_name,
 
 	parameters = callback_data;
 
-	file_path = medusa_full_path_from_directory_and_file
+	file_path = medusa_full_path_from_directory_and_file_name
 		(parameters->path, file_name);
 
 	for (p = keywords; p != NULL; p = p->next) {
@@ -740,7 +744,7 @@ add_private_keywords_to_index (const char *file_name,
 
 	parameters = callback_data;
 
-	file_path = medusa_full_path_from_directory_and_file
+	file_path = medusa_full_path_from_directory_and_file_name
 		(parameters->path, file_name);
 
 	for (p = keywords; p != NULL; p = p->next) {
@@ -771,7 +775,7 @@ index_public_metafile (MedusaMasterDB *db,
 	if (directory_path == NULL) {
 		return;
 	}
-	metafile_path = medusa_full_path_from_directory_and_file
+	metafile_path = medusa_full_path_from_directory_and_file_name
 		(directory_path, MEDUSA_PUBLIC_METAFILE_NAME);
 
 	parameters.db = db;
@@ -859,7 +863,7 @@ index_private_metafiles_for_user (MedusaMasterDB *db,
 	while ((file = readdir (dir)) != NULL) {
 		target_path = get_path_from_private_metafile_name (file->d_name);
 		if (target_path != NULL) {
-			metafile_path = medusa_full_path_from_directory_and_file
+			metafile_path = medusa_full_path_from_directory_and_file_name
 				(metafiles_directory_path, file->d_name);
 			index_private_metafile
 				(db, uid, metafile_path, target_path);
