@@ -24,15 +24,12 @@
 
 
 #include <config.h>
-#include "medusa-search-uri.h"
-
-#include "medusa-master-db-private.h"
-#include "medusa-uri-list.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
+#include <libmedusa/medusa-string.h>
 #include <libmedusa/medusa-utils.h>
 
 #include "medusa-master-db.h"
@@ -45,11 +42,12 @@
 static gboolean              bypass_search_method                         (char **string);
 static char *                get_next_root_uri                            (char **string);
 static char **               get_search_criteria                          (const char *string);
-static MedusaClauseClosure * clause_to_clause_closure                     (const char *clause,                       
-                                                                           MedusaMasterDB *master_db);
-static gboolean              clause_is_content_request                    (const char *clause);
-static char *                get_verb_from_clause                         (const char *clause);
-static char *                get_direct_object_from_clause                (const char *clause);
+static MedusaParsedSearchCriterion * parse_search_criterion               (const char *criterion,                       
+                                                                           MedusaMasterDB *master_db,
+                                                                           uid_t uid_of_user_making_query);
+static gboolean              criterion_is_content_request                 (const char *criterion);
+static char *                get_verb_from_criterion                      (const char *criterion);
+static char *                get_direct_object_from_criterion             (const char *criterion);
 static gboolean              request_wants_all_words_to_match             (const char *verb);
 static gboolean              request_is_for_positive_matches              (const char *verb);
 
@@ -59,40 +57,16 @@ static gboolean              search_uri_contains_size_requests            (const
 static char *                search_uri_add_user_can_read                 (const char *search_uri, 
                                                                            uid_t uid);
 static char *                search_uri_add_type_is_regular_file          (const char *search_uri);
-static void                  medusa_clause_closure_free_cover             (gpointer data,
-                                                                           gpointer user_data);
-static void                  medusa_clause_closure_free                   (MedusaClauseClosure *closure);
+static void                  medusa_parsed_search_criterion_free          (MedusaParsedSearchCriterion *parsed_search_criterion);
 
 static MedusaQueryOptimizationList *optimizations = NULL;
 
-static void
-print_optimization_result (MedusaOptimizationResult *optimization_result)
-{
-        int i;
 
-        if (optimization_result->syntax_error_found) {
-                g_print ("Syntax error found during optimization; aborting query.\n");
-                return;
-        }
-        if (optimization_result->query_is_always_true) {
-                g_print ("Query is TRUE in all cases\n");
-                return;
-        }
-        if (optimization_result->query_is_always_false) {
-                g_print ("Query is FALSEn");
-                return;
-        }
-        g_print ("The optimization produced a new query\n");
-        g_print ("search:[file:///]");
-        for (i = 0; optimization_result->criteria[i+1] != NULL; i++) {
-                g_print ("%s & ", optimization_result->criteria[i]);
-        }
-        g_print ("%s\n", optimization_result->criteria[i]);
-}
 
 MedusaParsedSearchURI *
-medusa_search_uri_parse (char *search_uri,
-                         MedusaMasterDB *master_db)
+medusa_search_uri_parse (const char *search_uri,
+                         MedusaMasterDB *master_db,
+                         uid_t uid_of_user_making_query)
 {
         char *location;
         char *root_uri;
@@ -101,18 +75,23 @@ medusa_search_uri_parse (char *search_uri,
         MedusaOptimizationResult *optimization_result;
         int i;
         MedusaParsedSearchURI *parsed_structure;
-        MedusaClauseClosure *clause_closure;
+        MedusaParsedSearchCriterion *parsed_search_criterion;
 
+        parsed_structure = g_new0 (MedusaParsedSearchURI, 1);
 
-
-        g_return_val_if_fail (medusa_uri_is_search_uri (search_uri), NULL);
-
+        if (!medusa_uri_is_search_uri (search_uri)) {
+                parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
+                return parsed_structure;
+        }
         
         location = strchr (search_uri, ':');
         location++;
         
         search_method_is_valid = bypass_search_method (&location);
-        g_return_val_if_fail (search_method_is_valid, NULL);
+        if (!search_method_is_valid) {
+                parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
+                return parsed_structure;
+        }
         
         root_uri = get_next_root_uri (&location);
 
@@ -122,7 +101,8 @@ medusa_search_uri_parse (char *search_uri,
         if (root_uri == NULL ||
             strcmp (root_uri, "file:///") ||
             get_next_root_uri (&location) != NULL) {
-                return NULL;
+                parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
+                return parsed_structure;
         }
         
         g_free (root_uri);
@@ -132,7 +112,8 @@ medusa_search_uri_parse (char *search_uri,
         
         /* For now we don't except or queries here */
         if (strchr (location, '|') != NULL) {
-                return NULL;
+                parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
+                return parsed_structure;
         }
 
         parsed_structure = g_new0 (MedusaParsedSearchURI, 1);        
@@ -140,7 +121,7 @@ medusa_search_uri_parse (char *search_uri,
         g_strdown (location);
         criteria = get_search_criteria (location);
         if  (criteria == NULL) {
-                parsed_structure->syntax_error_found = TRUE;
+                parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                 return parsed_structure;
         }
         if (optimizations == NULL) {
@@ -149,27 +130,22 @@ medusa_search_uri_parse (char *search_uri,
         optimization_result = medusa_query_optimizations_perform_and_free_deep (optimizations,
                                                                                 criteria);
 
-        print_optimization_result (optimization_result);
-        parsed_structure->query_is_always_true = optimization_result->query_is_always_true;
-        parsed_structure->query_is_always_false = optimization_result->query_is_always_false;
-        parsed_structure->syntax_error_found = optimization_result->syntax_error_found;
-
-        if (parsed_structure->query_is_always_true ||
-            parsed_structure->query_is_always_false ||
-            parsed_structure->syntax_error_found) {
+        parsed_structure->error = optimization_result->error;
+        if (parsed_structure->error != MEDUSA_SEARCH_URI_NO_ERROR) {
                 return parsed_structure;
         }
         
         for (i=0; optimization_result->criteria[i] != NULL; i++) {
-                clause_closure =  clause_to_clause_closure (optimization_result->criteria[i],
-                                                            master_db);
+                parsed_search_criterion =  parse_search_criterion (optimization_result->criteria[i],
+                                                                   master_db,
+                                                                   uid_of_user_making_query);
                 /* Detect syntax errors */
-                if (clause_closure == NULL) {
-                        parsed_structure->syntax_error_found = TRUE;
+                if (parsed_search_criterion == NULL) {
+                        parsed_structure->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                         medusa_optimization_result_destroy (optimization_result);
                         return parsed_structure;
                 }
-                parsed_structure->clause_closures = g_list_prepend (parsed_structure->clause_closures, clause_closure);
+                parsed_structure->parsed_search_criteria = g_list_prepend (parsed_structure->parsed_search_criteria, parsed_search_criterion);
                                                                
         }
         medusa_optimization_result_destroy (optimization_result);
@@ -183,50 +159,98 @@ medusa_parsed_search_uri_free (MedusaParsedSearchURI *parsed_search_uri)
         
 }
 
-void                     
-medusa_search_uri_parse_shutdown (void)
+void
+medusa_search_uri_clean_up_parse_data (void)
 {
-        if (optimizations != NULL) {
-                medusa_query_optimizations_destroy (optimizations);
-        }
+        medusa_query_optimizations_destroy (optimizations);
         optimizations = NULL;
 }
 
-static MedusaClauseClosure *
-clause_to_clause_closure (const char *clause,
-                          MedusaMasterDB *master_db)
+static MedusaDatabaseAndUIDData *
+database_and_uid_data_new (gpointer database,
+                           uid_t uid)
+{
+        MedusaDatabaseAndUIDData *data;
+        
+        data = g_new0 (MedusaDatabaseAndUIDData, 1);
+        data->database = database;
+        data->uid_of_user_making_query = uid;
+
+        return data;
+}
+
+static MedusaParsedSearchCriterion *
+parse_search_criterion (const char *criterion,
+                        MedusaMasterDB *master_db,
+                        uid_t uid_of_user_making_query)
 {
         MedusaQueryFunc evaluate;
-        MedusaClauseClosure *closure;
+        MedusaResultCheckFunc verify;
+        MedusaParsedSearchCriterion *closure;
         MedusaArgumentType type;
         char *verb, *direct_object;
 
-        verb = get_verb_from_clause (clause);
+        verb = get_verb_from_criterion (criterion);
         if (verb == NULL) {
                 return NULL;
         }
-        direct_object = get_direct_object_from_clause (clause);
+        direct_object = get_direct_object_from_criterion (criterion);
         if (direct_object == NULL) {
                 return NULL;
         }
-
-        evaluate = medusa_query_clauses_get_function (medusa_uri_list_get_query_clauses (master_db->uri_list),
-                                                      clause,
-                                                      &type);
-
+        closure = NULL;
+        medusa_query_clauses_get_function (medusa_uri_list_get_query_clauses (master_db->uri_list),
+                                           criterion,
+                                           &evaluate,
+                                           &verify,
+                                           &type);
         if (evaluate != NULL) {
-                closure = g_new0 (MedusaClauseClosure, 1);
+                closure = g_new0 (MedusaParsedSearchCriterion, 1);
                 closure->argument_type = type;
                 closure->query_func = evaluate;
-                
-                closure->uri_list = master_db->uri_list;
-                closure->file_system_db = NULL;
+                closure->check_func = verify;
+                closure->database_record_type = MEDUSA_URI_INDEX_RECORD;                
+                closure->query_data = master_db->uri_list;
+
+        }
+        
+        medusa_query_clauses_get_function (medusa_file_system_db_get_query_clauses_using_index_only (master_db->file_system_db),
+                                           criterion,
+                                           &evaluate,
+                                           &verify,
+                                           &type);
+        if (evaluate != NULL) {
+                closure = g_new0 (MedusaParsedSearchCriterion, 1);
+                closure->argument_type = type;
+                closure->query_func = evaluate;
+                closure->check_func = verify;
+                closure->query_data = master_db->file_system_db;
+                closure->database_record_type = MEDUSA_FILE_ATTRIBUTE_INDEX_RECORD;
+
+        }
+        medusa_query_clauses_get_function (medusa_file_system_db_get_query_clauses_using_index_and_uid (master_db->file_system_db),
+                                           criterion,
+                                           &evaluate,
+                                           &verify,
+                                           &type);
+        if (evaluate != NULL) {
+                closure = g_new0 (MedusaParsedSearchCriterion, 1);
+                closure->argument_type = type;
+                closure->query_func = evaluate;
+                closure->check_func = verify;
+                closure->query_data = database_and_uid_data_new (master_db->file_system_db,
+                                                                 uid_of_user_making_query);
+                closure->free_query_data_when_finished = TRUE;
+                closure->database_record_type = MEDUSA_FILE_ATTRIBUTE_INDEX_RECORD;
+
+        }
+        if (closure) {
                 switch (type) {
                 case MEDUSA_ARGUMENT_TYPE_STRING:
-                        closure->argument = g_strdup (direct_object);
+                        closure->argument.string = g_strdup (direct_object);
                         break;
-                case MEDUSA_ARGUMENT_TYPE_INTEGER:
-                        closure->argument = GINT_TO_POINTER(strtol (direct_object, NULL, 10));
+                case MEDUSA_ARGUMENT_TYPE_NUMBER:
+                        closure->argument.number = strtol (direct_object, NULL, 10);
                         break;
                 default:
                         g_assert_not_reached ();
@@ -237,34 +261,9 @@ clause_to_clause_closure (const char *clause,
                 g_free (direct_object);
                 return closure;
         }
-        
-        evaluate = medusa_query_clauses_get_function (medusa_file_system_db_get_query_clauses (master_db->file_system_db),
-                                                      clause,
-                                                      &type);
-        if (evaluate != NULL) {
-                closure = g_new0 (MedusaClauseClosure, 1);
-                closure->argument_type = type;
-                closure->query_func = evaluate;
-                closure->file_system_db = master_db->file_system_db;
-                closure->uri_list = NULL;
-                switch (type) {
-                case MEDUSA_ARGUMENT_TYPE_STRING:
-                        closure->argument = g_strdup (direct_object);
-                        break;
-                case MEDUSA_ARGUMENT_TYPE_INTEGER:
-                        closure->argument = GINT_TO_POINTER(strtol (direct_object, NULL, 10));
-                        break;
-                default:
-                        /* MEDUSA_ARGUMENT_TYPE_NONE */
-                        g_assert_not_reached ();
-                }
-                closure->is_content_request = FALSE;
-                g_free (verb);
-                g_free (direct_object);
-                return closure;
-        }
-        if (clause_is_content_request (clause)) {
-                closure = g_new0 (MedusaClauseClosure, 1);
+
+        if (criterion_is_content_request (criterion)) {
+                closure = g_new0 (MedusaParsedSearchCriterion, 1);
                 closure->argument_type = MEDUSA_ARGUMENT_TYPE_NONE;
                 closure->query_func = evaluate;
                 closure->is_content_request = TRUE;
@@ -365,52 +364,74 @@ search_uri_add_type_is_regular_file (const char *search_uri)
 }
 
 static void                  
-medusa_clause_closure_free_cover (gpointer data,
+parsed_search_criterion_free_cover (gpointer data,
                                   gpointer user_data)
 {
         g_return_if_fail (data != NULL);
 
-        medusa_clause_closure_free (data);
+        medusa_parsed_search_criterion_free (data);
 }
 
 static void                  
-medusa_clause_closure_free (MedusaClauseClosure *closure)
+medusa_parsed_search_criterion_free (MedusaParsedSearchCriterion *closure)
 {
         if (closure->argument_type == MEDUSA_ARGUMENT_TYPE_STRING) {
-                g_free (closure->argument);
+                g_free (closure->argument.string);
         }
 
         if (closure->is_content_request) {
                 g_free (closure->content_request);
         }
-        
+        if (closure->free_query_data_when_finished) {
+                g_free (closure->query_data);
+        }
         g_free (closure);
 }
 
 void
-medusa_clause_closure_list_free (GList *clause_closures)
+medusa_parsed_search_criterion_list_free (GList *parsed_search_criteria)
 {
-        g_list_foreach (clause_closures, medusa_clause_closure_free_cover, NULL);
-        g_list_free (clause_closures);
+        g_list_foreach (parsed_search_criteria, parsed_search_criterion_free_cover, NULL);
+        g_list_free (parsed_search_criteria);
 }
 
 gboolean              
-medusa_clause_closure_is_content_search (gpointer data,
+medusa_parsed_search_criterion_is_content_search (gpointer data,
                                          gpointer user_data)
 {
-        MedusaClauseClosure *clause_closure;
+        MedusaParsedSearchCriterion *parsed_search_criterion;
 
-        clause_closure = (MedusaClauseClosure *) data;
-        g_return_val_if_fail (clause_closure != NULL, FALSE);
+        parsed_search_criterion = (MedusaParsedSearchCriterion *) data;
+        g_return_val_if_fail (parsed_search_criterion != NULL, FALSE);
 
-        g_assert (clause_closure->is_content_request == TRUE ||
-                  clause_closure->is_content_request == FALSE);
+        g_assert (parsed_search_criterion->is_content_request == TRUE ||
+                  parsed_search_criterion->is_content_request == FALSE);
 
-        return clause_closure->is_content_request;
+        return parsed_search_criterion->is_content_request;
 }
 
+gboolean                 
+medusa_parsed_search_criterion_has_check (MedusaParsedSearchCriterion *criterion)
+{
+        g_return_val_if_fail (criterion != NULL, FALSE);
+
+        return criterion->check_func != NULL;
+}
+
+gboolean                 
+medusa_parsed_search_criterion_run_check (MedusaParsedSearchCriterion *criterion,
+                                          const char *uri)
+{
+        g_return_val_if_fail (criterion != NULL, FALSE);
+        g_return_val_if_fail (criterion->check_func != NULL, FALSE);
+
+        return criterion->check_func (uri,
+                                   criterion->argument);
+}
+
+
 gboolean
-medusa_uri_is_search_uri (char *uri)
+medusa_uri_is_search_uri (const char *uri)
 {
         char *trimmed_uri;
         gboolean result;
@@ -445,6 +466,11 @@ bypass_search_method (char **string)
                 *string += strlen ("index-only");
                 return TRUE;
         }
+        if (medusa_str_has_prefix (*string,
+                                   "index-if-available")) {
+                *string += strlen ("index-if-available");
+                return TRUE;
+        }                          
         if (medusa_str_has_prefix (*string,
                                    "index-with-backup")) {
                 *string += strlen ("index-with-backup");
@@ -494,40 +520,43 @@ get_search_criteria (const char *string)
 
 
 static gboolean              
-clause_is_content_request (const char *clause)
+criterion_is_content_request (const char *criterion)
 {
-        return medusa_str_has_prefix (clause, "content");
+        return medusa_str_has_prefix (criterion, "content");
 }
 
 static char *                
-get_verb_from_clause (const char *clause)
+get_verb_from_criterion (const char *criterion)
 {
         const char *end_of_word;
 
-        if (strchr (clause, ' ') == NULL) {
+        if (strchr (criterion, ' ') == NULL) {
                 return NULL;
         }
-        for ( ; *clause != ' '; clause++);
-        for ( ; *clause == ' '; clause++);
-        for (end_of_word = clause; *end_of_word != ' '; end_of_word++);
+        for ( ; *criterion != ' '; criterion++);
+        for ( ; *criterion == ' '; criterion++);
+        if (strchr (criterion, ' ') == NULL) {
+                return NULL;
+        }
+        for (end_of_word = criterion; *end_of_word != ' '; end_of_word++);
 
-        return g_strndup (clause, end_of_word - clause);
+        return g_strndup (criterion, end_of_word - criterion);
 
 }
 
 static char *                
-get_direct_object_from_clause (const char *clause)
+get_direct_object_from_criterion (const char *criterion)
 {
-        g_assert (strchr (clause, ' ') != NULL);
-        for ( ; *clause != ' '; clause++);
-        for ( ; *clause == ' '; clause++);
-        if (strchr (clause, ' ') == NULL) {
+        g_assert (strchr (criterion, ' ') != NULL);
+        for ( ; *criterion != ' '; criterion++);
+        for ( ; *criterion == ' '; criterion++);
+        if (strchr (criterion, ' ') == NULL) {
                 return NULL;
         }
-        for ( ; *clause != ' '; clause++);
-        for ( ; *clause == ' '; clause++);
+        for ( ; *criterion != ' '; criterion++);
+        for ( ; *criterion == ' '; criterion++);
 
-        return g_strdup (clause);
+        return g_strdup (criterion);
 
 }
 
