@@ -40,12 +40,26 @@
 # include <sys/param.h>
 # include <sys/ucred.h>
 # include <sys/mount.h>
-#elif defined(HAVE_GETMNTENT)
+#elif defined(HAVE_SYS_MNTTAB_H)
+# include <stdio.h>
+# include <sys/mnttab.h>
+#else
+#if defined(HAVE_GETMNTENT)
 # include <mntent.h>
 #else
 #warning "Can't find a valid mount function type"
 #endif
+#endif
 
+#ifdef HAVE_STATVFS
+# include <sys/statvfs.h>
+#elif HAVE_STATFS
+# include <sys/statfs.h>
+#else
+# warning "Medusa will not be able to record your indexing progress because you don't have either statfs or statvfs."
+#endif
+
+/* FIXME: What is this for? */
 #ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
@@ -94,7 +108,7 @@ medusa_index_progress_new (char *root_directory)
         output_fd = open (INDEX_PROGRESS_FILE_NAME, O_RDWR | O_CREAT | O_EXCL | O_SYNC);
         g_return_val_if_fail (output_fd != -1, NULL);
 
-        chmod (INDEX_PROGRESS_FILE_NAME, S_IRWXU | S_IRWXG | S_IRWXO);
+        chmod (INDEX_PROGRESS_FILE_NAME, S_IRWXU | S_IRGRP | S_IROTH);
 
         index_progress = g_new0 (MedusaIndexProgress, 1);
         index_progress->total_blocks_to_index = total_blocks_to_index;
@@ -171,7 +185,7 @@ medusa_index_progress_destroy (MedusaIndexProgress *progress)
 gboolean
 medusa_index_is_currently_running (void)
 {
-        return (medusa_index_service_is_available () == MEDUSA_INDEXER_ERROR_BUSY);
+       return medusa_index_service_is_available () == MEDUSA_INDEXER_ALREADY_INDEXING;
 }
           
 int
@@ -181,7 +195,9 @@ medusa_index_progress_get_percentage_complete (void)
         int percentage_complete;
         int read_result;
         
-        g_return_val_if_fail (medusa_index_is_currently_running (), -1);
+        if (medusa_index_is_currently_running () == FALSE) {
+                return 0;
+        }
 
         progress_fd = open (INDEX_PROGRESS_FILE_NAME, O_RDONLY);
         if (progress_fd == -1) {
@@ -203,19 +219,30 @@ get_disk_used_information (char *root_directory,
                            size_t *total_blocks_to_index,
                            size_t *block_size)
 {
-        struct statfs statfs_buffer;
-        int statfs_result;
+#ifdef HAVE_STATVFS
+        struct statvfs stat_buffer;
+#elif defined(HAVE_STATFS)
+        struct statfs stat_buffer;
+#endif
+        int stat_result;
        
         g_print ("Root directory is %s\n", root_directory);
-        statfs_result = statfs (root_directory, &statfs_buffer);
+#ifdef HAVE_STATVFS
+        stat_result = statvfs (root_directory, &stat_buffer);
+#elif defined(HAVE_STATFS)
+        stat_result = statfs (root_directory, &stat_buffer);
+#else
+        stat_result = -1;
+#endif
+
         /* Don't return disk used information for directories that are not
            the root of file systems */
-        if (statfs_result != 0) {
+        if (stat_result != 0) {
                 /* FIXME bugzilla.eazel.com 4557: log that we are doing this */
                 return FALSE;
         }
-        *block_size = statfs_buffer.f_bsize; 
-        *total_blocks_to_index = statfs_buffer.f_blocks;
+        *block_size = stat_buffer.f_bsize; 
+        *total_blocks_to_index = stat_buffer.f_blocks;
         return TRUE;
 }
 
