@@ -27,13 +27,12 @@
 /* medusa-index-service.c -- API for externals users of the medusa
    indexing service. */
 
+#include <config.h>
 #include "medusa-index-service.h"
 #include "medusa-index-service-private.h"
 #include "medusa-log.h"
 #include "medusa-service-private.h"
 
-#include <libmedusa-internal/medusa-conf.h>
-#include <libmedusa-internal/medusa-index-filenames.h>
 
 #include <glib.h>
 #include <stdlib.h>
@@ -145,7 +144,8 @@ get_last_index_update_time_by_hack (void)
         int stat_return_code;
         char *location_file_name; 
 
-        location_file_name = g_strdup_printf ("%s%s%s", MEDUSA_PREFIX, "/share/medusa/", "text-index-location-file");
+        location_file_name = g_strdup_printf ("%s/%s", MEDUSA_LOCALSTATEDIR,
+				       "text-index-location-file");
         stat_return_code = stat (location_file_name, &index_file_info);
         if (stat_return_code != 0) {
                 return 0;
