@@ -246,15 +246,21 @@ medusa_unindexed_search_is_available_for_uri (const char *uri)
         return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
 }
 
-static void
+static gboolean
 cleanup_current_directory_information (MedusaUnindexedSearch *search)
 {
+        
+        if (search->directory_stream_stack == NULL) {
+                return FALSE;
+        }
+
         closedir (search->directory_stream_stack->data);
         keywords_information_destroy (search->keywords_stack->data);
         g_free (search->directory_name_stack->data);
         
         pop_keyword_information_and_directory_from_stacks (search);
 
+        return search->directory_stream_stack != NULL;
 }
 
 GnomeVFSResult         
@@ -281,22 +287,17 @@ medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
         /* If we didn't find anything in the current directory,
            go up the stack frame until we find a place with results,
            or no more results */
-        cleanup_current_directory_information (search);
-        while (*result_uri == NULL &&
-               search->directory_name_stack) {
+        while (*result_uri == NULL) {
+                if (!cleanup_current_directory_information (search)) {
+                        return GNOME_VFS_ERROR_EOF;
+                }
                 *result_uri = search_directory_contents_in_progress (search, context, &search_was_cancelled);
-                cleanup_current_directory_information (search);
                 if (search_was_cancelled) {
                         return GNOME_VFS_ERROR_CANCELLED;
                 }
         }
-        
-        if (*result_uri == NULL) {
-                return GNOME_VFS_ERROR_EOF;
-        }
-        else {
-                return GNOME_VFS_OK;
-        }
+
+        return GNOME_VFS_OK;
 }
 
 static void
@@ -439,7 +440,6 @@ search_directory (MedusaUnindexedSearch *search,
         if (medusa_is_unsearched_location (directory_uri)) {
                 return NULL;
         }
-        
 
         directory_path = gnome_vfs_get_local_path_from_uri (directory_uri);
         g_return_val_if_fail (directory_path != NULL, NULL);
@@ -478,7 +478,7 @@ search_directory (MedusaUnindexedSearch *search,
             search_result != NULL) {
                 return search_result;
         }
-
+        
         cleanup_current_directory_information (search);
 
         return NULL;
